from django.utils.translation import ugettext_lazy as _
from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register, ModelAdminGroup
from .models import GalleryPage, Category


class GalleryPageModelAdmin(ModelAdmin):
    """Makes the adding of gallery pages easy by providing a hook to the menu system thus not needing to use the page hierarchy"""
    model = GalleryPage
    menu_label = _('Gallery')
    menu_icon = 'picture'
    list_display = ('first_published_at', 'title', 'description', 'categories')
    list_filter = ('first_published_at', 'categories')
    search_fields = ('title', 'description', 'first_published_at')


class GalleryCategoryAdmin(ModelAdmin):
    """Makes the adding of categories simply by giving own menu entry"""
    model = Category
    menu_label = _('Categories')
    menu_icon = 'group'
    list_display = ('name',)
    search_fields = ('name',)


class GalleryModelAdminGroup(ModelAdminGroup):
    """Creates the side menu entry with the two other ModelAdmins inside of it"""
    menu_label = _('Gallery')
    menu_icon = 'picture'
    menu_order = 300
    items = (GalleryPageModelAdmin, GalleryCategoryAdmin)


modeladmin_register(GalleryModelAdminGroup)



from django.db import models
from wagtail.core.models import Page, Orderable
from wagtail.admin.edit_handlers import RichTextField, FieldPanel, InlinePanel, MultiFieldPanel
from wagtail.images.edit_handlers import ImageChooserPanel
from modelcluster.fields import ParentalKey
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from wagtail.contrib.routable_page.models import RoutablePageMixin, route
from django.shortcuts import render
from .managers import CategoryManager
from django.core.exceptions import ValidationError
from django.template.defaultfilters import slugify
from wagtail.search import index
from django.utils.translation import ugettext_lazy as _
import django.http


# Create your models here.

class GalleryParentPage(RoutablePageMixin, Page):
    """
    Root page to which all galleries are attached
    """

    body = RichTextField(blank=True, help_text=_('Text to appear on gallery root page'), verbose_name=_('Body'))
    """ Text that accompanies that appears on root gallery page"""

    content_panels = Page.content_panels + [
        FieldPanel('body', classname="full"),
    ]


    show_in_menus = True
    subpage_types = ['wagtail_gallery.GalleryPage']

    search_fields = Page.search_fields + [
        index.SearchField('body')
    ]

    class Meta:
        verbose_name = _("Gallery Root Page")

    def get_context(self, request: django.http.HttpRequest, *args, **kwargs: dict):
        """
        Overrides get_context of Page model and adds the archives and galleries to the context

        :param request: Django HttpRequest
        :param args: Request args
        :param kwargs: Request kwargs
        :return: Django context
        """
        context = super(GalleryParentPage, self).get_context(request)

        context['archives'] = self.get_archives()
        context['galleries'] = self.get_pagination(request, kwargs)

        return context

    def get_pagination(self, request: django.http.HttpRequest, kwargs) -> django.core.paginator.Paginator:
        """
        Get pagination for the galleries parent page

        :param request: Django HttpRequest
        :param kwargs: Request kwargs
        :return: Pagination object
        """
        if ('year' in kwargs) and ('month' in kwargs):
            galleries = GalleryPage.objects.live().filter(first_published_at__year=kwargs['year'],
                                                          first_published_at__month=kwargs['month']).order_by(
                '-first_published_at')
        elif 'year' in kwargs:
            galleries = GalleryPage.objects.live().filter(first_published_at__year=kwargs['year']).order_by(
                '-first_published_at')
        else:
            galleries = self.get_children().live().order_by('-first_published_at')
        paginator = Paginator(galleries, 10)  # Show 10 resources per page

        page = request.GET.get('page')
        try:
            galleries = paginator.page(page)
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            galleries = paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999), deliver last page of results.
            galleries = paginator.page(paginator.num_pages)

        return galleries

    def get_pagination_category(self, request: django.http.HttpRequest, kwargs) -> django.core.paginator.Paginator:
        """
        Get pagination for the galleries parent page, but filtered by category

        :param request: Django HttpRequest
        :param kwargs: Request kwargs
        :return: Pagination object
        """
        if ('year' in kwargs) and ('month' in kwargs):
            galleries = GalleryPage.objects.live().filter(categories__name__icontains=kwargs['category'])
            galleries = galleries.filter(first_published_at__year=kwargs['year'],
                                         first_published_at__month=kwargs['month']).order_by(
                '-first_published_at')
        elif 'year' in kwargs:
            galleries = GalleryPage.objects.live().filter(categories__name__icontains=kwargs['category'])
            galleries = galleries.filter(first_published_at__year=kwargs['year']).order_by(
                '-first_published_at')
        else:
            galleries = GalleryPage.objects.live().filter(categories__name__icontains=kwargs['category'])
            galleries = galleries.order_by('-first_published_at')
        paginator = Paginator(galleries, 10)  # Show 10 resources per page

        page = request.GET.get('page')
        try:
            galleries = paginator.page(page)
        except PageNotAnInteger:
            # If page is not an integer, deliver first page.
            galleries = paginator.page(1)
        except EmptyPage:
            # If page is out of range (e.g. 9999), deliver last page of results.
            galleries = paginator.page(paginator.num_pages)

        return galleries

    @property
    def get_galleries(self) -> models.QuerySet:
        """
        Gets the galleries that are the children of a specific parent page

        :return: Queryset containing Gallery objects
        """
        galleries = GalleryParentPage.get_children().live()
        return galleries

    @property
    def get_categories(self) -> models.QuerySet:
        """
        Gets the categories available

        :return: Queryset consisting of gallery objects
        """
        return Category.objects.all()

    @property
    def get_gallery_base_url(self) -> str:
        """
        URL of the base gallery parent page

        :return: URL string
        """
        return self.url

    def get_archives(self) -> list:
        """
        Gets the archives of the gallery page objects

        :return: List of dictionaries containing [{'date': <GalleryPage object>]}
        """
        archives = list(GalleryPage.objects.live().dates('first_published_at', 'month', order='DESC'))
        archives = [{'date': archive} for archive in archives]
        return archives

    def get_archives_category(self, kwargs: dict) -> list:
        """
       Gets the archives of the gallery page objects, but filtered for a specific category

       :return: List of dictionaries containing [{'date': <GalleryPage object>]}
       """
        archives = GalleryPage.objects.live().filter(categories__name__icontains=kwargs['category'])
        archives = list(archives.dates('first_published_at', 'month', order='DESC'))
        archives = [{'date': archive} for archive in archives]
        return archives

    @route(r"^(?P<year>(?:19|20)\d\d)/(?P<month>1[012]|0[1-9])/$")
    def archiveMonth(self, request: django.http.HttpRequest, **kwargs: dict) -> django.http.HttpResponse:
        """
        Route that returns archive page for a given year and month that contains said year and months gallery
        pages

        :param request: Django HttpRequest
        :param kwargs: Request kwargs
        :return: HttpResponse page that shows said archives
        """
        return render(request, "wagtail_gallery/gallery_parent_page.html",
                      {'self': self, 'page': self, 'galleries': self.get_pagination(request, kwargs),
                       'archives': self.get_archives()})

    @route(r"^(?P<year>(?:19|20)\d\d)/$")
    def archiveYear(self, request: django.http.HttpRequest, **kwargs: dict) -> django.http.HttpResponse:
        """
        Route that returns archive page for a given year that contains said year and months
        gallery pages

        :param request: Django HttpRequest
        :param kwargs: Request kwargs
        :return:  HttpResponse page that shows said archives
        """
        return render(request, "wagtail_gallery/gallery_parent_page.html",
                      {'self': self, 'page': self, 'galleries': self.get_pagination(request, kwargs),
                       'archives': self.get_archives()})

    @route(r"^category/(?P<category>[\w\-]+)/$")
    def archive(self, request: django.http.HttpRequest, **kwargs: dict) -> django.http.HttpResponse:
        """
        Routing for a particular category

        :param request: Django HttpRequest
        :param kwargs: Request kwargs
        :return: HttpResponse
        """
        return render(request, "wagtail_gallery/gallery_parent_page.html",
                      {'self': self, 'page': self, 'galleries': self.get_pagination_category(request, kwargs),
                       'archives': self.get_archives_category(kwargs)})

    @route(r"^category/(?P<category>[\w\-]+)/(?P<year>(?:19|20)\d\d)/(?P<month>1[012]|0[1-9])/$")
    def archiveCategoryMonth(self, request: django.http.HttpRequest, **kwargs: dict) -> django.http.HttpResponse:
        """
        Routing for a particular category filtered for a particular year and month

        :param request: Django HttpRequest
        :param kwargs: Request kwargs
        :return: HttpResponse
        """
        return render(request, "wagtail_gallery/gallery_parent_page.html",
                      {'self': self, 'page': self, 'galleries': self.get_pagination_category(request, kwargs),
                       'archives': self.get_archives_category(kwargs)})

    @route(r"^category/(?P<category>[\w\-]+)/(?P<year>(?:19|20)\d\d)/$")
    def archiveCategoryYear(self, request: django.http.HttpRequest, **kwargs: dict) -> django.http.HttpResponse:
        """
        Routing for a particular category filtered only by year

        :param request: Django HttpRequest
        :param kwargs: Request kwargs
        :return: HttpResponse
        """
        return render(request, "wagtail_gallery/gallery_parent_page.html",
                      {'self': self, 'page': self, 'galleries': self.get_pagination_category(request, kwargs),
                       'archives': self.get_archives_category(kwargs)})


class GalleryPage(Page):
    body = RichTextField(blank=True, help_text=_('Body of text on page'), verbose_name=_('Body'))
    """Optional body of text that is shown on a specifc gallery page"""

    description = models.TextField(blank=True, null=True, help_text=_('Short description of gallery'),
                                   verbose_name=_('Description'))
    """Optional short description of the gallery used when showing cards etc."""

    categories = models.ManyToManyField('wagtail_gallery.Category', through='wagtail_gallery.CategoryGalleryPage', blank=True,
                                        help_text=_('Categories relevant to gallery'), verbose_name=_('Categories'))
    """Categories to which a particular gallery belongs"""

    content_panels = Page.content_panels + [
        FieldPanel('body', classname="full"),
        FieldPanel('description'),
        MultiFieldPanel([
            InlinePanel("gallery_categories", label=_("Categories"))
        ]),
        InlinePanel('gallery_image', label=_("Gallery image"),
                    panels=[
                        FieldPanel('description'),
                        ImageChooserPanel('image')]
                    )
    ]

    show_in_menus = False
    subpage_types = []
    parent_page_types = ["wagtail_gallery.GalleryParentPage"]

    search_fields = Page.search_fields + [
        index.SearchField('body'),
        index.SearchField('description'),
        index.RelatedFields('categories', [
            index.SearchField('name'),
            index.SearchField('description'),
        ])
    ]

    class Meta:
        verbose_name = _("Gallery")

    def get_context(self, request: django.http.HttpRequest, *args, **kwargs: dict):
        """
        Gets the context for a specific gallery page

        :param request: Django HttpRequest
        :param args: Request args
        :param kwargs: Request kwargs
        :return: Context variable
        """
        context = super(GalleryPage, self).get_context(request)

        context['archives'] = self.get_archives()

        return context

    def get_archives(self) -> list:
        """
        Gets the archives for the gallery

        :return: List of dictionaries of the form: [{'date': <GalleryPage Queryset>}]
        """
        archives = list(GalleryPage.objects.live().dates('first_published_at', 'month', order='DESC'))
        archives = [{'date': archive} for archive in archives]
        return archives

    @property
    def get_categories(self) -> models.QuerySet:
        """
        Gets the categories available to all galleries

        :return: Queryset containing Category objects
        """
        return Category.objects.all()

    @property
    def get_gallery_base_url(self) -> str:
        """
        Gets the url of the root gallery page to which this gallery is attached

        :return: URL string
        """
        return self.get_parent().url


class GalleryImage(Orderable):
    """
    A particular image that has an order component to it. Used to allow ordering of the gallery images on a specific gallery page
    """

    gallery = ParentalKey(GalleryPage, related_name='gallery_image')
    """Required. Not manually entered. A gallery to which to the object belongs"""

    description = models.TextField(blank=True, null=True, verbose_name=_('Description'))
    """Optional description of a particular image. Used as a caption for said image."""

    image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=False,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_('Image')
    )
    """Actual image to be used for said GalleryImage object"""

    content_panels = Page.content_panels + [
        FieldPanel('description'),
        ImageChooserPanel('image')
    ]


class Category(models.Model):
    """
    Category to which a gallery page may belong
    """

    name = models.CharField(max_length=80, unique=True, verbose_name=_('Category name'))
    """Required. Name of the category."""

    slug = models.SlugField(unique=True, max_length=80)
    """Required. Not currently used. Slug of the category. I recommend making same as name."""

    parent = models.ForeignKey(
        'self',
        blank=True,
        null=True,
        related_name="children",
        verbose_name=_('Parent category'),
        on_delete=models.SET_NULL
    )
    """Optional. Parent of the category. Not currently used but allows for sub-categories."""

    description = models.CharField(max_length=500, blank=True, verbose_name=_('Description'))
    """Description of a category. Not currently used."""

    objects = CategoryManager()

    def __str__(self):
        return self.name

    def clean(self):
        """
        Method that overrides models.Model clean method that ensures no circular references are made when it comes to the parent.
        """
        if self.parent:
            parent = self.parent
            if self.parent == self:
                raise ValidationError(_('Parent category cannot be self.'))
            if parent.parent and parent.parent == self:
                raise ValidationError(_('Cannot have circular Parents.'))

    def save(self, *args, **kwargs):
        """
        Method that overrides models.Model save method so as to deal with the slug
        """
        if not self.slug:
            self.slug = slugify(self.name)
        return super(Category, self).save(*args, **kwargs)

    class Meta:
        ordering = ['name']
        verbose_name = _("Gallery Category")
        verbose_name_plural = _("Gallery Categories")


class CategoryGalleryPage(models.Model):
    """Used internally. Ignore"""

    category = models.ForeignKey(Category, related_name="+", verbose_name=_('Category'), on_delete=models.CASCADE)
    page = ParentalKey('GalleryPage', related_name='gallery_categories')
    panels = [
        FieldPanel('category')
    ]

    def __str__(self):
        return str(self.category)



# Generated by Django 2.0.9 on 2018-10-05 18:19

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields
import wagtail.contrib.routable_page.models
import wagtail.core.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailimages', '0021_image_file_hash'),
        ('wagtailcore', '0040_page_draft_title'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=80, unique=True, verbose_name='Category name')),
                ('slug', models.SlugField(max_length=80, unique=True)),
                ('description', models.CharField(blank=True, max_length=500, verbose_name='Description')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='children', to='wagtail_gallery.Category', verbose_name='Parent category')),
            ],
            options={
                'verbose_name': 'Gallery Category',
                'verbose_name_plural': 'Gallery Categories',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='CategoryGalleryPage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtail_gallery.Category', verbose_name='Category')),
            ],
        ),
        migrations.CreateModel(
            name='GalleryImage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort_order', models.IntegerField(blank=True, editable=False, null=True)),
                ('description', models.TextField(blank=True, null=True, verbose_name='Description')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='GalleryPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('body', wagtail.core.fields.RichTextField(blank=True, help_text='Body of text on page', verbose_name='Body')),
                ('description', models.TextField(blank=True, help_text='Short description of gallery', null=True, verbose_name='Description')),
                ('categories', models.ManyToManyField(blank=True, help_text='Categories relevant to gallery', through='wagtail_gallery.CategoryGalleryPage', to='wagtail_gallery.Category', verbose_name='Categories')),
            ],
            options={
                'verbose_name': 'Gallery',
            },
            bases=('wagtailcore.page',),
        ),
        migrations.CreateModel(
            name='GalleryParentPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('body', wagtail.core.fields.RichTextField(blank=True, help_text='Text to appear on gallery root page', verbose_name='Body')),
            ],
            options={
                'verbose_name': 'Gallery Root Page',
            },
            bases=(wagtail.contrib.routable_page.models.RoutablePageMixin, 'wagtailcore.page'),
        ),
        migrations.AddField(
            model_name='galleryimage',
            name='gallery',
            field=modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='gallery_image', to='wagtail_gallery.GalleryPage'),
        ),
        migrations.AddField(
            model_name='galleryimage',
            name='image',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailimages.Image', verbose_name='Image'),
        ),
        migrations.AddField(
            model_name='categorygallerypage',
            name='page',
            field=modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='gallery_categories', to='wagtail_gallery.GalleryPage'),
        ),
    ]



from django.db import models


class CategoryManager(models.Manager):

    def with_uses(self, gallery_page):
        entries = gallery_page.get_entries()
        return self.filter(gallerypage__in=entries).distinct()



import os

from setuptools import find_packages, setup

NAME = 'wagtail_gallery'
DESCRIPTION = 'A simple gallery app built for Wagtail CMS and Django'
URL = 'https://gitlab.com/dfmeyer/wagtail_gallery'
EMAIL = 'me3064@gmail.com'
AUTHOR = 'Daniel F. Meyer'
REQUIRES_PYTHON = '>=3.5'
VERSION = '0.1'
LICENSE = 'MIT'
KEYWORDS = ['Django', 'Wagtail', 'gallery']
PROJECT_URLS = {
    "Bug Tracker": "https://gitlab.com/dfmeyer/wagtail_gallery/issues",
    "Documentation": "https://wagtail-gallery.readthedocs.io/en/latest/",
    "Source Code": "https://gitlab.com/dfmeyer/wagtail_gallery",
}

REQUIRED = ['wagtail', 'django-social-share', ]

here = os.path.abspath(os.path.dirname(__file__))

try:
    with open('README.rst') as f:
        LONG_DESCRIPTION = f.read()
except FileNotFoundError:
    LONG_DESCRIPTION = DESCRIPTION

setup(
    name=NAME,
    version=VERSION,
    packages=find_packages(),
    include_package_data=True,
    url=URL,
    license=LICENSE,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    long_description = LONG_DESCRIPTION,
    long_description_content_type='text/x-rst',
    python_requires=REQUIRES_PYTHON,
    install_requires=REQUIRED,
    keywords=KEYWORDS,
    project_urls=PROJECT_URLS,
)
